///////////////////////////////////////////////////
// DIPSwitch.cpp
// part of
// DIPSwitch
// 720020-1178x
//
// Applied Data Systems
//
// See DIPSwitch.h for a description and history
// 
///////////////////////////////////////////////////
#include "DIPSwitch.h"

#define SA1111_BASE		(0x0C000000)
#define BX_DIP_BASE		(SA1111_BASE + 0x1000)
#define BX_DIP_OFFSET	(0x14)
#define	BX_DIP_MASK		(0xC)
#define BX_DIP_SHIFT	(0x2)

#define PAGE_SIZE 				0x1000 	// 1 page:4KB
#define PAGE_ALIGN(Vaddr) 		((PVOID)((ULONG)(Vaddr) & ~(PAGE_SIZE - 1)))
#define ROUND_TO_PAGES(Size)  	(((ULONG)(Size) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

extern "C"
{
   __declspec(dllimport) BOOL   VirtualCopy(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);
}

// forward declarations
PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize);

BitsyXSwitch::BitsyXSwitch() : DIPSwitch(BX_DIP_BASE, BX_DIP_OFFSET, BX_DIP_MASK, BX_DIP_SHIFT)
{
}

DIPSwitch::DIPSwitch(DWORD address, DWORD offset, DWORD mask, DWORD shift)
{
	m_address = (void*)address;
	m_mask = mask;
	m_shift = shift;
	m_offset = (offset / sizeof(DWORD));
}

BYTE DIPSwitch::ReadSwitches()
{
	DWORD *p = (DWORD*)MapPhysicalAddr(m_address, PAGE_SIZE);

	DWORD val = p[m_offset];
	val &= m_mask;
	val = val >> m_shift;

	VirtualFree(p, 0, MEM_RELEASE);

	return(BYTE)val;
}


PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize)
{
	LPVOID pVirtualAddr;
	PVOID PageAlinedPhysicalAddress;
	ULONG Offset;
	DWORD PageAlignedSize;
	BOOL bCopyReturn;

	// Get Page Aligned Address
	PageAlinedPhysicalAddress = PAGE_ALIGN (dwPhysicalAddress);
	Offset = (ULONG)dwPhysicalAddress - (ULONG)PageAlinedPhysicalAddress;

	// Get page Aligned Size
	PageAlignedSize = ROUND_TO_PAGES(dwSize + Offset);

	// Create reserved block of virtual memory
	pVirtualAddr = VirtualAlloc(0, PageAlignedSize, MEM_RESERVE, PAGE_NOACCESS);

    if(pVirtualAddr == NULL) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : Error Return. \r\n"));
        return NULL;
    }

	// Map physical memory to virtual memory
 	// then /256 and use the PAGE_PHYSICAL flag
	//----------------------------------------------
    bCopyReturn = VirtualCopy (	pVirtualAddr, 
								(LPVOID)((ULONG)PageAlinedPhysicalAddress >> 8),
								PageAlignedSize, 
								PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL);

    if(bCopyReturn == FALSE) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : return false from VirtualCopy(). \r\n"));      
        VirtualFree( pVirtualAddr, 0, MEM_RELEASE  );
        return NULL;
    }

	pVirtualAddr = (PVOID)((PUCHAR)pVirtualAddr + Offset);

	return pVirtualAddr;

}
